# src\file_conversor\backend\office\writer_backend.py

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.office.abstract_libreoffice_backend import AbstractLibreofficeBackend

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class LibreofficeWriterBackend(AbstractLibreofficeBackend):
    """
    A class that provides an interface for handling doc files using ``writer`` (libreoffice).
    """

    SUPPORTED_IN_FORMATS = {
        "doc": {},
        "docx": {},
        "odt": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "doc": {},
        "docx": {},
        "odt": {},
        "pdf": {},
        "html": {},
    }

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(verbose=verbose)
